@@ -2,10 +2,10 @@
 from pathlib import Path
 
 from .toc import update_indexname, add_toctree
-from .directive.toc import TableofContents
+from .directive.toc import TableofContents, SwapTableOfContents
 
 
-__version__ = "0.7.1"
+__version__ = "0.7.2"
 
 
 def add_static_files(app, config):
@@ -37,6 +37,9 @@ def setup(app):
     # Directives
     app.add_directive("tableofcontents", TableofContents)
 
+    # Transforms
+    app.add_post_transform(SwapTableOfContents)
+
     return {
         "version": __version__,
         "parallel_read_safe": True,
